@@ -53,6 +53,9 @@
 /* autoconfiguration */
 #include "config.h"
 
+/* version */
+#include "fsl_version.c"
+
 /* pcre static vector size */
 #define OVECSIZE 30
 
@@ -70,9 +73,6 @@
 #define CU(returncode) STMT( rc = returncode; goto CUS; )
 #define VCU STMT( goto CUS; )
 
-/* prefix of configuration files */
-#define FSL_PREFIX "fsl."
-
 /* general return codes */
 typedef enum {
     FSL_OK = 0,               /* everything ok */
@@ -81,7 +81,8 @@ typedef enum {
     FSL_ERR_USE,              /* invalid use */
     FSL_ERR_MEM,              /* no more memory available */
     FSL_ERR_SYS,              /* operating system error, see errno */
-    FSL_ERR_CUS               /* error handled and logged, just run clean up sequence and goodbye */
+    FSL_ERR_CUS,              /* error handled and logged, just run clean up sequence and goodbye */
+    FSL_ERR_INT               /* internal error */
 } fsl_rc_t;
 
 /* file buffer handling sub-API */
@@ -182,7 +183,9 @@ static struct {
     int           maskpri;
     int           logopt;
     int           delayopen;
-    int           openlog;
+    int           triedopenlog;
+    int           triedopenfsldebug;
+    int           openfsldebugpermanenterror;
 } ctx = { 
     NULL,
     NULL,
@@ -192,6 +195,8 @@ static struct {
     LOG_UPTO(LOG_DEBUG),
     0,
     FALSE,
+    FALSE,
+    FALSE,
     FALSE
 };
 
@@ -269,24 +274,32 @@ static fsl_rc_t readallfiles(buf_t *buffer)
     DIR *dp = NULL;
     struct dirent *de;
     char *filename = NULL;
+    char *cfgdir;
+    char *prefix;
     int n;
 
     if (buffer == NULL)
         CU(FSL_ERR_ARG);
-    fsldebug(L2_LEVEL_TRACE, "readallfiles() globbing \"%s/%s*\"", FSL_CFGDIR, FSL_PREFIX);
 
-    if ((dp = opendir(FSL_CFGDIR)) == NULL)
+    if ((cfgdir = getenv("FSL_CFGDIR")) == NULL)
+        cfgdir = FSL_CFGDIR;
+    if ((prefix = getenv("FSL_PREFIX")) == NULL)
+        prefix = FSL_PREFIX;
+
+    fsldebug(L2_LEVEL_TRACE, "readallfiles() globbing \"%s/%s*\"", cfgdir, prefix);
+
+    if ((dp = opendir(cfgdir)) == NULL)
         CU(FSL_ERR_SYS);
 
     rc = FSL_ERR_ARG;
     while ((de = readdir(dp)) != NULL) {
         n = strlen(de->d_name);
-        if (   (n >= strlen(FSL_PREFIX))
-            && (strncmp(de->d_name, FSL_PREFIX, strlen(FSL_PREFIX)) == 0)) {
-            if ((filename = (char *)malloc(strlen(FSL_CFGDIR) + 1 + n + 1)) == NULL)
+        if (   (n >= strlen(prefix))
+            && (strncmp(de->d_name, prefix, strlen(prefix)) == 0)) {
+            if ((filename = (char *)malloc(strlen(cfgdir) + 1 + n + 1)) == NULL)
                 CU(FSL_ERR_MEM);
             filename[0] = '\0';
-            strcat(filename, FSL_CFGDIR);
+            strcat(filename, cfgdir);
             strcat(filename, "/");
             strcat(filename, de->d_name);
             if (appendfiletobuffer(buffer, filename) == FSL_OK)
@@ -595,6 +608,68 @@ CUS:
     return rc;
 }
 
+static void closefsllog()
+{
+    if (ctx.l2_fslnch != NULL) {
+        l2_channel_destroy(ctx.l2_fslnch);
+        ctx.l2_fslnch = NULL;
+    }
+    if (ctx.l2_fslenv != NULL) {
+        l2_env_destroy(ctx.l2_fslenv);
+        ctx.l2_fslenv = NULL;
+    }
+    ctx.triedopenfsldebug = FALSE;
+}
+
+static void openfsldebug()
+{
+    int rc = FSL_OK;
+    char *argl2spec;
+    l2_result_t   l2rv;
+    char *cp; /* scratch variable */
+    l2_channel_t *ch; /* scratch variable */
+
+    if (ctx.triedopenfsldebug || ctx.openfsldebugpermanenterror)
+        return;
+
+    /* properly prepare for repeated execution */
+    ctx.triedopenfsldebug = TRUE;
+
+    /* create OSSP l2 environment for fsl itself (internal logging) */
+    if ((argl2spec = getenv("FSL_DEBUG")) == NULL)
+        argl2spec = FSL_DEBUG;
+    if (strlen(argl2spec) > 0) {
+        if ((l2rv = l2_env_create(&ctx.l2_fslenv)) != L2_OK) {
+            cp = l2_env_strerror(ctx.l2_fslenv, l2rv); fsldebug(L2_LEVEL_ERROR, "openlog: failed to create L2 environment (%d) for fsl", l2rv); CU(FSL_ERR_INT); }
+        if ((l2rv = l2_env_levels(ctx.l2_fslenv, L2_LEVEL_ALL, L2_LEVEL_NONE)) != L2_OK) {
+            cp = l2_env_strerror(ctx.l2_fslenv, l2rv); fsldebug(L2_LEVEL_ERROR, "openlog: logging failed to set global logging level defaults %s(%d) for fsl", cp, l2rv); CU(FSL_ERR_INT); }
+        if ((l2rv = l2_env_formatter(ctx.l2_fslenv, 'D', l2_util_fmt_dump, NULL)) != L2_OK) {
+            cp = l2_env_strerror(ctx.l2_fslenv, l2rv); fsldebug(L2_LEVEL_ERROR, "openlog: logging failed to register dump formatter %s(%d) for fsl", cp, l2rv); CU(FSL_ERR_INT); }
+        if ((l2rv = l2_env_formatter(ctx.l2_fslenv, 'S', l2_util_fmt_string, NULL)) != L2_OK) {
+            cp = l2_env_strerror(ctx.l2_fslenv, l2rv); fsldebug(L2_LEVEL_ERROR, "openlog: logging failed to register string formatter %s(%d) for fsl", cp, l2rv); CU(FSL_ERR_INT); }
+        if ((l2rv = l2_env_formatter(ctx.l2_fslenv, 'm', l2_util_fmt_errno, NULL)) != L2_OK) {
+            cp = l2_env_strerror(ctx.l2_fslenv, l2rv); fsldebug(L2_LEVEL_ERROR, "openlog: logging failed to register errno formatter %s(%d) for fsl", cp, l2rv); CU(FSL_ERR_INT); }
+        if ((l2rv = l2_channel_create(&ctx.l2_fslnch, ctx.l2_fslenv, "noop")) != L2_OK) {
+            cp = l2_env_strerror(ctx.l2_fslenv, l2rv); fsldebug(L2_LEVEL_ERROR, "openlog: logging failed to create noop channel; %s(%d) for fsl", cp, l2rv); CU(FSL_ERR_INT); }
+        if ((l2rv = l2_spec(&ch, ctx.l2_fslenv, "%s", argl2spec)) != L2_OK) {
+            cp = l2_env_strerror(ctx.l2_fslenv, l2rv); fsldebug(L2_LEVEL_ERROR, "openlog: logging failed to create stream from spec %s(%d) for fsl", cp, l2rv);
+            if (l2rv == L2_ERR_ARG)
+                CU(FSL_ERR_ARG);
+            else
+                CU(FSL_ERR_INT);
+        }
+        if ((l2rv = l2_channel_link(ctx.l2_fslnch, L2_LINK_CHILD, ch, NULL)) != L2_OK) {
+            cp = l2_env_strerror(ctx.l2_fslenv, l2rv); fsldebug(L2_LEVEL_ERROR, "openlog: logging failed to link child channel %s(%d) for fsl", cp, l2rv); CU(FSL_ERR_INT); }
+        if ((l2rv = l2_channel_open(ctx.l2_fslnch)) != L2_OK) {
+            cp = l2_env_strerror(ctx.l2_fslenv, l2rv); fsldebug(L2_LEVEL_ERROR, "openlog: logging failed to open channel stream %s(%d) for fsl", cp, l2rv); CU(FSL_ERR_INT); }
+    }
+CUS:
+    if (rc != FSL_OK)
+        closefsllog();
+    if (rc == FSL_ERR_ARG)
+        ctx.openfsldebugpermanenterror = TRUE;
+}
+
 /* POSIX API function openlog(3) */
 void openlog(const char *ident, int logopt, int facility)
 {
@@ -603,13 +678,11 @@ void openlog(const char *ident, int logopt, int facility)
     fsl_rc_t rv;
     cfg_t           *cfg;
     cfg_rc_t         cfgrv;
-    char *argl2spec;
     char *cp; /* scratch variable */
     int   i;  /* scratch variable */
     char *cpIdent;
     char *cpFacility;
     char *cpISF;
-    l2_channel_t *ch; /* scratch variable */
     l2_result_t   l2rv;
 
     /* initialization */
@@ -620,37 +693,15 @@ void openlog(const char *ident, int logopt, int facility)
     cpIdent = NULL;
     cpISF = NULL;
 
-    /* properly handle repeated execution */
+    /* properly prepare for repeated execution */
     closelog();
-    ctx.openlog = TRUE;
+    ctx.triedopenlog = TRUE;
 
     /* create OSSP l2 environment for fsl itself (internal logging) */
-    if ((argl2spec = getenv("FSL_DEBUG")) == NULL)
-        argl2spec = FSL_DEBUG;
-    if (strlen(argl2spec) > 0) {
-        if ((l2rv = l2_env_create(&ctx.l2_fslenv)) != L2_OK) {
-            cp = l2_env_strerror(ctx.l2_fslenv, l2rv); fsldebug(L2_LEVEL_ERROR, "openlog: failed to create L2 environment (%d) for fsl", l2rv); CU(1); }
-        if ((l2rv = l2_env_levels(ctx.l2_fslenv, L2_LEVEL_ALL, L2_LEVEL_NONE)) != L2_OK) {
-            cp = l2_env_strerror(ctx.l2_fslenv, l2rv); fsldebug(L2_LEVEL_ERROR, "openlog: logging failed to set global logging level defaults %s(%d) for fsl", cp, l2rv); CU(1); }
-        if ((l2rv = l2_env_formatter(ctx.l2_fslenv, 'D', l2_util_fmt_dump, NULL)) != L2_OK) {
-            cp = l2_env_strerror(ctx.l2_fslenv, l2rv); fsldebug(L2_LEVEL_ERROR, "openlog: logging failed to register dump formatter %s(%d) for fsl", cp, l2rv); CU(1); }
-        if ((l2rv = l2_env_formatter(ctx.l2_fslenv, 'S', l2_util_fmt_string, NULL)) != L2_OK) {
-            cp = l2_env_strerror(ctx.l2_fslenv, l2rv); fsldebug(L2_LEVEL_ERROR, "openlog: logging failed to register string formatter %s(%d) for fsl", cp, l2rv); CU(1); }
-        if ((l2rv = l2_env_formatter(ctx.l2_fslenv, 'm', l2_util_fmt_errno, NULL)) != L2_OK) {
-            cp = l2_env_strerror(ctx.l2_fslenv, l2rv); fsldebug(L2_LEVEL_ERROR, "openlog: logging failed to register errno formatter %s(%d) for fsl", cp, l2rv); CU(1); }
-        if ((l2rv = l2_channel_create(&ctx.l2_fslnch, ctx.l2_fslenv, "noop")) != L2_OK) {
-            cp = l2_env_strerror(ctx.l2_fslenv, l2rv); fsldebug(L2_LEVEL_ERROR, "openlog: logging failed to create noop channel; %s(%d) for fsl", cp, l2rv); CU(1); }
-        if ((l2rv = l2_spec(&ch, ctx.l2_fslenv, "%s", argl2spec)) != L2_OK) {
-            cp = l2_env_strerror(ctx.l2_fslenv, l2rv); fsldebug(L2_LEVEL_ERROR, "openlog: logging failed to create stream from spec %s(%d) for fsl", cp, l2rv); CU(1); }
-        if ((l2rv = l2_channel_link(ctx.l2_fslnch, L2_LINK_CHILD, ch, NULL)) != L2_OK) {
-            cp = l2_env_strerror(ctx.l2_fslenv, l2rv); fsldebug(L2_LEVEL_ERROR, "openlog: logging failed to link child channel %s(%d) for fsl", cp, l2rv); CU(1); }
-        if ((l2rv = l2_channel_open(ctx.l2_fslnch)) != L2_OK) {
-            cp = l2_env_strerror(ctx.l2_fslenv, l2rv); fsldebug(L2_LEVEL_ERROR, "openlog: logging failed to open channel stream %s(%d) for fsl", cp, l2rv); CU(1); }
-    }
+    openfsldebug();
 
     /* tracing */
-    
-    fsldebug(L2_LEVEL_TRACE, "openlog() ident=\"%s\", logopt=0x%.8lx, facility=0x%.8lx)", ident, logopt, facility);
+    fsldebug(L2_LEVEL_TRACE, "openlog() ident=\"%s\", logopt=0x%.8lx, facility=0x%.8lx; caught by FSL %s", ident, logopt, facility, fsl_version.v_gnu);
 
     /* remember logopt and handle unsupported values */
     ctx.logopt = logopt;
@@ -758,14 +809,7 @@ CUS:
 void closelog(void)
 {
     fsldebug(L2_LEVEL_TRACE, "closelog()");
-    if (ctx.l2_fslnch != NULL) {
-        l2_channel_destroy(ctx.l2_fslnch);
-        ctx.l2_fslnch = NULL;
-    }
-    if (ctx.l2_fslenv != NULL) {
-        l2_env_destroy(ctx.l2_fslenv);
-        ctx.l2_fslenv = NULL;
-    }
+    closefsllog();
     if (ctx.l2_nch != NULL) {
         l2_channel_destroy(ctx.l2_nch);
         ctx.l2_nch = NULL;
@@ -781,7 +825,7 @@ void closelog(void)
     ctx.maskpri = LOG_UPTO(LOG_DEBUG);
     ctx.logopt = 0;
     ctx.delayopen = FALSE;
-    ctx.openlog = FALSE;
+    ctx.triedopenlog = FALSE;
     return;
 }
 
@@ -822,7 +866,7 @@ void vsyslog(int priority, const char *fmt, va_list args)
     char *cp;
     
     /* check for previous omitted attempt of initialization */
-    if (ctx.l2_nch == NULL && !ctx.openlog)
+    if (ctx.l2_nch == NULL && !ctx.triedopenlog)
         openlog("fsl", 0, LOG_SYSLOG);
 
     /* check for previous proper initialization */
